@@ -7,7 +7,7 @@
  * This file is part of Libgcrypt.
  *
  * Libgcrypt is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
+ * it under the terms of the GNU Lesser general Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
@@ -54,7 +54,7 @@ static unsigned int debug_flags;
 static int force_fips_mode;
 
 /* Controlled by global_init().  */
-int _gcry_global_any_init_done;
+static int any_init_done;
 
 /*
  * Functions called before and after blocking syscalls.
@@ -91,9 +91,9 @@ global_init (void)
 {
   gcry_error_t err = 0;
 
-  if (_gcry_global_any_init_done)
+  if (any_init_done)
     return;
-  _gcry_global_any_init_done = 1;
+  any_init_done = 1;
 
   /* Tell the random module that we have seen an init call.  */
   _gcry_set_preferred_rng_type (0);
@@ -102,9 +102,6 @@ global_init (void)
   if (!pre_syscall_func)
     gpgrt_get_syscall_clamp (&pre_syscall_func, &post_syscall_func);
 
-  /* Add a handler to be called after log_fatal and log_debug.  */
-  _gcry_set_gpgrt_post_log_handler ();
-
   /* See whether the system is in FIPS mode.  This needs to come as
      early as possible but after ATH has been initialized.  */
   _gcry_initialize_fips_mode (force_fips_mode);
@@ -143,25 +140,6 @@ global_init (void)
   BUG ();
 }
 
-#ifdef ENABLE_HMAC_BINARY_CHECK
-# if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 7 )
-# define GCC_ATTR_CONSTRUCTOR  __attribute__ ((__constructor__))
-
-static void GCC_ATTR_CONSTRUCTOR
-_gcry_global_constructor (void)
-{
-  force_fips_mode = _gcry_fips_to_activate ();
-  if (force_fips_mode)
-    {
-      no_secure_memory = 1;
-      global_init ();
-      _gcry_fips_run_selftests (0);
-      _gcry_random_close_fds ();
-      no_secure_memory = 0;
-    }
-}
-# endif
-#endif /* ENABLE_HMAC_BINARY_CHECK */
 
 /* This function is called by the macro fips_is_operational and makes
    sure that the minimal initialization has been done.  This is far
@@ -183,7 +161,7 @@ _gcry_global_constructor (void)
 int
 _gcry_global_is_operational (void)
 {
-  if (!_gcry_global_any_init_done)
+  if (!any_init_done)
     {
 #ifdef HAVE_SYSLOG
       syslog (LOG_USER|LOG_WARNING, "Libgcrypt warning: "
@@ -283,8 +261,7 @@ _gcry_check_version (const char *req_version)
     /* Compare version numbers.  */
     if ( my_major > rq_major
 	|| (my_major == rq_major && my_minor > rq_minor)
-	|| (my_major == rq_major && my_minor == rq_minor
-	    && my_micro > rq_micro)
+	|| (my_major == rq_major && my_minor == rq_minor		                           		 && my_micro > rq_micro)
 	|| (my_major == rq_major && my_minor == rq_minor
                                  && my_micro == rq_micro))
       {
@@ -310,7 +287,12 @@ print_config (const char *what, gpgrt_stream_t fp)
   if (!what || !strcmp (what, "cc"))
     {
       gpgrt_fprintf (fp, "cc:%d:%s:\n",
-                     GPGRT_GCC_VERSION,
+#if GPGRT_VERSION_NUMBER >= 0x011b00 /* 1.27 */
+                     GPGRT_GCC_VERSION
+#else
+                     _GPG_ERR_GCC_VERSION /* Due to a bug in gpg-error.h.  */
+#endif
+                     ,
 #ifdef __clang__
                      "clang:" __VERSION__
 #elif __GNUC__
@@ -334,11 +316,8 @@ print_config (const char *what, gpgrt_stream_t fp)
 #if USE_RNDEGD
                      "egd:"
 #endif
-#if USE_RNDGETENTROPY
-                     "getentropy:"
-#endif
-#if USE_RNDOLDLINUX
-                     "oldlinux:"
+#if USE_RNDLINUX
+                     "linux:"
 #endif
 #if USE_RNDUNIX
                      "unix:"
@@ -354,11 +333,6 @@ print_config (const char *what, gpgrt_stream_t fp)
       gpgrt_fprintf (fp, "cpu-arch:"
 #if defined(HAVE_CPU_ARCH_X86)
                      "x86"
-#              ifdef __x86_64__
-                     ":amd64"
-#              else
-                     ":i386"
-#              endif
 #elif defined(HAVE_CPU_ARCH_ALPHA)
                      "alpha"
 #elif defined(HAVE_CPU_ARCH_SPARC)
@@ -378,9 +352,6 @@ print_config (const char *what, gpgrt_stream_t fp)
   if (!what || !strcmp (what, "mpi-asm"))
     gpgrt_fprintf (fp, "mpi-asm:%s:\n", _gcry_mpi_get_hw_config ());
 
-  if (!what || !strcmp (what, "mpi-pow"))
-    gpgrt_fprintf (fp, "mpi-powm:%s\n", _gcry_mpi_get_powm_config ());
-
   if (!what || !strcmp (what, "hwflist"))
     {
       unsigned int hwfeatures, afeature;
@@ -397,19 +368,10 @@ print_config (const char *what, gpgrt_stream_t fp)
     {
       /* We use y/n instead of 1/0 for the stupid reason that
        * Emacsen's compile error parser would accidentally flag that
-       * line when printed during "make check" as an error.  The
-       * second field is obsolete and thus empty (used to be used for
-       * a so-called enforced-fips-mode).  The third field has an
-       * option static string describing the module versions; this is
-       * an optional configure option.  */
-      gpgrt_fprintf (fp, "fips-mode:%c::%s:\n",
+       * line when printed during "make check" as an error.  */
+      gpgrt_fprintf (fp, "fips-mode:%c:%c:\n",
                      fips_mode ()? 'y':'n',
-#ifdef FIPS_MODULE_VERSION
-                     fips_mode () ? FIPS_MODULE_VERSION : ""
-#else
-                     ""
-#endif /* FIPS_MODULE_VERSION */
-                     );
+                     _gcry_enforced_fips_mode ()? 'y':'n' );
     }
 
   if (!what || !strcmp (what, "rng-type"))
@@ -429,21 +391,6 @@ print_config (const char *what, gpgrt_stream_t fp)
       jver = _gcry_rndjent_get_version (&active);
       gpgrt_fprintf (fp, "rng-type:%s:%d:%u:%d:\n", s, i, jver, active);
     }
-
-  if (!what || !strcmp (what, "compliance"))
-    {
-      /* Right now we have no certification for 1.9 so we return an
-       * empty string.  As soon as this version has been approved for
-       * VS-Nfd we will put the string "de-vs" into the second
-       * field. If further specifications are required they are added
-       * as parameters to that field.  Other certifications will go
-       * into field 3 and so on.
-       *  field 1: keyword "compliance"
-       *  field 2: German VS-Nfd is marked with "de-vs"
-       *  field 3: reserved for FIPS.
-       */
-      gpgrt_fprintf (fp, "compliance:%s::\n", "");
-    }
 }
 
 
@@ -471,13 +418,6 @@ _gcry_get_config (int mode, const char *what)
     return NULL;
 
   print_config (what, fp);
-
-  if (!what)
-    {
-      /* Null-terminate bulk output. */
-      gpgrt_fwrite ("\0", 1, 1, fp);
-    }
-
   if (gpgrt_ferror (fp))
     {
       save_errno = errno;
@@ -513,11 +453,6 @@ _gcry_get_config (int mode, const char *what)
 
 
 
-#if _GCRY_GCC_VERSION >= 40200
-# pragma GCC diagnostic push
-# pragma GCC diagnostic ignored "-Wswitch"
-#endif
-
 /* Command dispatcher function, acting as general control
    function.  */
 gcry_err_code_t
@@ -529,7 +464,7 @@ _gcry_vcontrol (enum gcry_ctl_cmds cmd, va_list arg_ptr)
   switch (cmd)
     {
     case GCRYCTL_ENABLE_M_GUARD:
-      rc = GPG_ERR_NOT_SUPPORTED;
+      _gcry_private_enable_m_guard ();
       break;
 
     case GCRYCTL_ENABLE_QUICK_RANDOM:
@@ -563,9 +498,7 @@ _gcry_vcontrol (enum gcry_ctl_cmds cmd, va_list arg_ptr)
 
     case GCRYCTL_DISABLE_SECMEM:
       global_init ();
-      /* When FIPS enabled, no effect at all.  */
-      if (!fips_mode ())
-        no_secure_memory = 1;
+      no_secure_memory = 1;
       break;
 
     case GCRYCTL_INIT_SECMEM:
@@ -598,10 +531,6 @@ _gcry_vcontrol (enum gcry_ctl_cmds cmd, va_list arg_ptr)
 			       & ~GCRY_SECMEM_FLAG_SUSPEND_WARNING));
       break;
 
-    case GCRYCTL_AUTO_EXPAND_SECMEM:
-      _gcry_secmem_set_auto_expand (va_arg (arg_ptr, unsigned int));
-      break;
-
     case GCRYCTL_USE_SECURE_RNDPOOL:
       global_init ();
       _gcry_secure_random_alloc (); /* Put random number into secure memory. */
@@ -637,7 +566,7 @@ _gcry_vcontrol (enum gcry_ctl_cmds cmd, va_list arg_ptr)
       break;
 
     case GCRYCTL_ANY_INITIALIZATION_P:
-      if (_gcry_global_any_init_done)
+      if (any_init_done)
 	rc = GPG_ERR_GENERAL;
       break;
 
@@ -691,11 +620,16 @@ _gcry_vcontrol (enum gcry_ctl_cmds cmd, va_list arg_ptr)
       break;
 
     case GCRYCTL_SET_RANDOM_DAEMON_SOCKET:
-      rc = GPG_ERR_NOT_SUPPORTED;
+      _gcry_set_preferred_rng_type (0);
+      _gcry_set_random_daemon_socket (va_arg (arg_ptr, const char *));
       break;
 
     case GCRYCTL_USE_RANDOM_DAEMON:
-      rc = GPG_ERR_NOT_SUPPORTED;
+      /* We need to do make sure that the random pool is really
+         initialized so that the poll function is not a NOP. */
+      _gcry_set_preferred_rng_type (0);
+      _gcry_random_initialize (1);
+      _gcry_use_random_daemon (!! va_arg (arg_ptr, int));
       break;
 
     case GCRYCTL_CLOSE_RANDOM_DEVICE:
@@ -732,7 +666,9 @@ _gcry_vcontrol (enum gcry_ctl_cmds cmd, va_list arg_ptr)
       break;
 
     case GCRYCTL_FIPS_MODE_P:
-      if (fips_mode ())
+      if (fips_mode ()
+          && !_gcry_is_fips_mode_inactive ()
+          && !no_secure_memory)
 	rc = GPG_ERR_GENERAL; /* Used as TRUE value */
       break;
 
@@ -742,7 +678,7 @@ _gcry_vcontrol (enum gcry_ctl_cmds cmd, va_list arg_ptr)
          selftest is triggered.  It is not possible to put the libraty
          into fips mode after having passed the initialization. */
       _gcry_set_preferred_rng_type (0);
-      if (!_gcry_global_any_init_done)
+      if (!any_init_done)
         {
           /* Not yet initialized at all.  Set a flag so that we are put
              into fips mode during initialization.  */
@@ -760,27 +696,6 @@ _gcry_vcontrol (enum gcry_ctl_cmds cmd, va_list arg_ptr)
       }
       break;
 
-    case GCRYCTL_NO_FIPS_MODE:
-      /* Performing this command puts the library into non-fips mode,
-         even if system has fips setting.  It is not possible to put
-         the libraty into non-fips mode after having passed the
-         initialization. */
-      _gcry_set_preferred_rng_type (0);
-      if (!_gcry_global_any_init_done)
-        {
-          /* Not yet initialized at all.  Set a flag so that we are put
-             into non-fips mode during initialization.  */
-          force_fips_mode = 0;
-        }
-      else if (!init_finished)
-        {
-          /* Already initialized.  */
-          _gcry_no_fips_mode_required = 1;
-        }
-      else
-	rc = GPG_ERR_GENERAL;
-      break;
-
     case GCRYCTL_SELFTEST:
       /* Run a selftest.  This works in fips mode as well as in
          standard mode.  In contrast to the power-up tests, we use an
@@ -790,58 +705,10 @@ _gcry_vcontrol (enum gcry_ctl_cmds cmd, va_list arg_ptr)
       rc = _gcry_fips_run_selftests (1);
       break;
 
-    case GCRYCTL_FIPS_SERVICE_INDICATOR:
-      rc = _gcry_fips_indicator ();
-      break;
-
-    case GCRYCTL_FIPS_REJECT_NON_FIPS:
-      {
-        unsigned int flags = va_arg (arg_ptr, unsigned int);
-        _gcry_thread_context_set_reject (flags);
-      }
-      break;
-
-    case GCRYCTL_FIPS_SERVICE_INDICATOR_CIPHER:
-      /* Get FIPS Service Indicator for a given symmetric algorithm and
-       * optional mode. Returns GPG_ERR_NO_ERROR if algorithm is allowed or
-       * GPG_ERR_NOT_SUPPORTED otherwise */
-      rc = _gcry_fips_indicator_cipher (arg_ptr);
-      break;
-
-    case GCRYCTL_FIPS_SERVICE_INDICATOR_MAC:
-      /* Get FIPS Service Indicator for a given message authentication code.
-       * Returns GPG_ERR_NO_ERROR if algorithm is allowed or
-       * GPG_ERR_NOT_SUPPORTED otherwise */
-      rc = _gcry_fips_indicator_mac (arg_ptr);
-      break;
-
-    case GCRYCTL_FIPS_SERVICE_INDICATOR_MD:
-      /* Get FIPS Service Indicator for a given message digest. Returns
-       * GPG_ERR_NO_ERROR if algorithm is allowed or GPG_ERR_NOT_SUPPORTED
-       * otherwise */
-      rc = _gcry_fips_indicator_md (arg_ptr);
-      break;
-
-    case GCRYCTL_FIPS_SERVICE_INDICATOR_KDF:
-      /* Get FIPS Service Indicator for a given KDF. Returns GPG_ERR_NO_ERROR
-       * if algorithm is allowed or GPG_ERR_NOT_SUPPORTED otherwise */
-      rc = _gcry_fips_indicator_kdf (arg_ptr);
-      break;
-
-    case GCRYCTL_FIPS_SERVICE_INDICATOR_FUNCTION:
-      /* Get FIPS Service Indicator for a given function from the API.
-       * Returns GPG_ERR_NO_ERROR if the function is allowed or
-       * GPG_ERR_NOT_SUPPORTED otherwise */
-      rc = _gcry_fips_indicator_function (arg_ptr);
-      break;
-
-    case GCRYCTL_FIPS_SERVICE_INDICATOR_PK_FLAGS:
-      /* Get FIPS Service Indicator for a public key operation flags.
-       * Returns GPG_ERR_NO_ERROR if the flag is allowed to be used or
-       * GPG_ERR_NOT_SUPPORTED otherwise */
-      rc = _gcry_fips_indicator_pk_flags (arg_ptr);
-      break;
-
+#if _GCRY_GCC_VERSION >= 40600
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wswitch"
+#endif
     case PRIV_CTL_INIT_EXTRNG_TEST:  /* Init external random test.  */
       rc = GPG_ERR_NOT_SUPPORTED;
       break;
@@ -866,6 +733,9 @@ _gcry_vcontrol (enum gcry_ctl_cmds cmd, va_list arg_ptr)
     case PRIV_CTL_DUMP_SECMEM_STATS:
       _gcry_secmem_dump_stats (1);
       break;
+#if _GCRY_GCC_VERSION >= 40600
+# pragma GCC diagnostic pop
+#endif
 
     case GCRYCTL_DISABLE_HWF:
       {
@@ -875,7 +745,14 @@ _gcry_vcontrol (enum gcry_ctl_cmds cmd, va_list arg_ptr)
       break;
 
     case GCRYCTL_SET_ENFORCED_FIPS_FLAG:
-      /* Obsolete - ignore  */
+      if (!any_init_done)
+        {
+          /* Not yet initialized at all.  Set the enforced fips mode flag */
+          _gcry_set_preferred_rng_type (0);
+          _gcry_set_enforced_fips_mode ();
+        }
+      else
+        rc = GPG_ERR_GENERAL;
       break;
 
     case GCRYCTL_SET_PREFERRED_RNG_TYPE:
@@ -892,7 +769,7 @@ _gcry_vcontrol (enum gcry_ctl_cmds cmd, va_list arg_ptr)
       {
         int *ip = va_arg (arg_ptr, int*);
         if (ip)
-          *ip = _gcry_get_rng_type (!_gcry_global_any_init_done);
+          *ip = _gcry_get_rng_type (!any_init_done);
       }
       break;
 
@@ -920,8 +797,7 @@ _gcry_vcontrol (enum gcry_ctl_cmds cmd, va_list arg_ptr)
         int npers = va_arg (arg_ptr, int);
         if (va_arg (arg_ptr, void *) || npers < 0)
           rc = GPG_ERR_INV_ARG;
-        else if (_gcry_get_rng_type (!_gcry_global_any_init_done)
-                 != GCRY_RNG_TYPE_FIPS)
+        else if (_gcry_get_rng_type (!any_init_done) != GCRY_RNG_TYPE_FIPS)
           rc = GPG_ERR_NOT_SUPPORTED;
         else
           rc = _gcry_rngdrbg_reinit (flagstr, pers, npers);
@@ -941,9 +817,6 @@ _gcry_vcontrol (enum gcry_ctl_cmds cmd, va_list arg_ptr)
   return rc;
 }
 
-#if _GCRY_GCC_VERSION >= 40200
-# pragma GCC diagnostic pop
-#endif
 
 
 /* Set custom allocation handlers.  This is in general not useful
@@ -961,11 +834,10 @@ _gcry_set_allocation_handler (gcry_handler_alloc_t new_alloc_func,
 
   if (fips_mode ())
     {
-      /* In FIPS mode, we can not use custom allocation handlers because
-       * fips requires explicit zeroization and we can not guarantee that
-       * with custom free functions (and we can not do it transparently as
-       * in free we do not know the zize). */
-      return;
+      /* We do not want to enforce the fips mode, but merely set a
+         flag so that the application may check whether it is still in
+         fips mode.  */
+      _gcry_inactivate_fips_mode ("custom allocation handler");
     }
 
   alloc_func = new_alloc_func;
@@ -996,13 +868,30 @@ _gcry_set_outofcore_handler (int (*f)(void*, size_t, unsigned int), void *value)
 {
   global_init ();
 
-  if (fips_mode ())
-    return;
+  if (fips_mode () )
+    {
+      log_info ("out of core handler ignored in FIPS mode\n");
+      return;
+    }
 
   outofcore_handler = f;
   outofcore_handler_value = value;
 }
 
+/* Return the no_secure_memory flag.  */
+static int
+get_no_secure_memory (void)
+{
+  if (!no_secure_memory)
+    return 0;
+  if (_gcry_enforced_fips_mode ())
+    {
+      no_secure_memory = 0;
+      return 0;
+    }
+  return no_secure_memory;
+}
+
 
 static gcry_err_code_t
 do_malloc (size_t n, unsigned int flags, void **mem)
@@ -1010,7 +899,7 @@ do_malloc (size_t n, unsigned int flags, void **mem)
   gcry_err_code_t err = 0;
   void *m;
 
-  if ((flags & GCRY_ALLOC_FLAG_SECURE) && !no_secure_memory)
+  if ((flags & GCRY_ALLOC_FLAG_SECURE) && !get_no_secure_memory ())
     {
       if (alloc_secure_func)
 	m = (*alloc_secure_func) (n);
@@ -1069,13 +958,27 @@ _gcry_malloc_secure (size_t n)
 int
 _gcry_is_secure (const void *a)
 {
-  if (no_secure_memory)
+  if (get_no_secure_memory ())
     return 0;
   if (is_secure_func)
     return is_secure_func (a) ;
   return _gcry_private_is_secure (a);
 }
 
+void
+_gcry_check_heap( const void *a )
+{
+  (void)a;
+
+    /* FIXME: implement this*/
+#if 0
+    if( some_handler )
+	some_handler(a)
+    else
+	_gcry_private_check_heap(a)
+#endif
+}
+
 static void *
 _gcry_realloc_core (void *a, size_t n, int xhint)
 {
@@ -1125,7 +1028,7 @@ _gcry_free (void *p)
   else
     _gcry_private_free (p);
 
-  if (save_errno && save_errno != errno)
+  if (save_errno)
     gpg_err_set_errno (save_errno);
 }
 
