@@ -91,7 +91,7 @@ typedef struct pooldesc_s
 static pooldesc_t mainpool;
 
 
-/* A couple of flags whith some beeing set early. */
+/* A couple of flags whith some being set early. */
 static int disable_secmem;
 static int show_warning;
 static int not_locked;
@@ -454,7 +454,7 @@ init_pool (pooldesc_t *pool, size_t n)
 
   /* Initialize first memory block.  */
   mb = (memblock_t *) pool->mem;
-  mb->size = pool->size;
+  mb->size = pool->size - BLOCK_HEAD_SIZE;
   mb->flags = 0;
 }
 
@@ -610,7 +610,7 @@ _gcry_secmem_malloc_internal (size_t size, int xhint)
   mb = mb_get_new (pool, (memblock_t *) pool->mem, size);
   if (mb)
     {
-      stats_update (pool, size, 0);
+      stats_update (pool, mb->size, 0);
       return &mb->aligned.c;
     }
 
@@ -624,7 +624,7 @@ _gcry_secmem_malloc_internal (size_t size, int xhint)
           mb = mb_get_new (pool, (memblock_t *) pool->mem, size);
           if (mb)
             {
-              stats_update (pool, size, 0);
+              stats_update (pool, mb->size, 0);
               return &mb->aligned.c;
             }
         }
@@ -641,7 +641,7 @@ _gcry_secmem_malloc_internal (size_t size, int xhint)
         return NULL; /* Not enough memory available for a new pool.  */
       /* Initialize first memory block.  */
       mb = (memblock_t *) pool->mem;
-      mb->size = pool->size;
+      mb->size = pool->size - BLOCK_HEAD_SIZE;
       mb->flags = 0;
 
       pool->okay = 1;
@@ -660,7 +660,7 @@ _gcry_secmem_malloc_internal (size_t size, int xhint)
       mb = mb_get_new (pool, (memblock_t *) pool->mem, size);
       if (mb)
         {
-          stats_update (pool, size, 0);
+          stats_update (pool, mb->size, 0);
           return &mb->aligned.c;
         }
     }
@@ -841,6 +841,8 @@ _gcry_secmem_term ()
 }
 
 
+/* Print stats of the secmem allocator.  With EXTENDED passwed as true
+ * a detiled listing is returned (used for testing).  */
 void
 _gcry_secmem_dump_stats (int extended)
 {
@@ -872,6 +874,5 @@ _gcry_secmem_dump_stats (int extended)
                       mb->size);
         }
     }
-
   SECMEM_UNLOCK;
 }
